#!/usr/bin/perl
# 
# sb::Mailer - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Mailer;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.00';
# 0.00 [2005/03/02] porting from sbmail.pl

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Language ();
# ==================================================
# // constructor
# ==================================================
sub new {
	my $class = shift;
	my $self  = {
		'default_user' => 'anonymous@example.com',
		'sender'       => 'sendmail',
		'send_server'  => '/usr/sbin/sendmail',
		'charcode'     => 'jis',
		'pop_server'   => undef,
		'pop_account'  => undef,
		'pop_password' => undef,
		'pop_useapop'  => 0,
		@_,
	};
	sb::Language->get->checkcode('',$self->{'charcode'});
	return bless($self,$class);
}
# ==================================================
# // public functions
# ==================================================
sub sendmail { # ᡼
	my $self = shift;
	my %param = (
		'To'      => undef,
		'From'    => $self->{'default_user'},
		'Subject' => 'no title',
		'Body'    => '',
		@_,
	);
	eval {
		die("no destination address\n") if (!$param{'To'});
		my $head = &_generate_header(%param);
		$param{'Body'} = sb::Language->get->mailtext($param{'Body'});
		if ($self->{'sender'} eq 'sendmail') {
			my $sendmail = $self->{'send_server'};
			my $from = $param{'From'};
			die("cannot use sendmail, please check the path for sendmail\n") if !(-x $sendmail);
			open(SENDMAIL,"| $sendmail -t -i -f '$from'") or die("cannot open via sendmail\n");
			print SENDMAIL $head;
			print SENDMAIL $param{'Body'};
			close(SENDMAIL);
		} elsif ($self->{'sender'} eq 'smtp') {
			require Net::SMTP;
			my $smtp = Net::SMTP->new(
				$self->{'send_server'},
				'Hello'=>$self->{'send_server'}
			);
			$smtp->mail($param{'From'});
			$smtp->to($param{'To'});
			$smtp->data();
			$smtp->datasend($head);
			$smtp->datasend($param{'Body'});
			$smtp->dataend();
			$smtp->quit;
		}
	};
	return ($@) ? $@ : undef; # Ԥ硢顼å֤
}
sub receive { # ᡼
	my $self = shift;
	my %param = ( # ϥѥ᡼
		'subj'  => undef, # ᡼ Subject 
		'msgid' => undef, # Ѥߥ᡼ Message-ID å
		'from'  => undef, # ᡼ From  (Υե)
		'size'  => undef, # ƥ᡼륵 ( -1 ˤȥ¤ʤ)
		'del'   => undef, # ᡼򥵡Ф뤫ɤ / 0 : Ĥ, 1 : 
		@_,
	);
	my @mail; # ϥѥ᡼
	if ( !defined($self->{'pop_server'}) 
	  or !defined($self->{'pop_account'}) 
	  or !defined($self->{'pop_password'}) ) {
		die('Lack of parameters');
	}
	eval {
		require Net::POP3;
		my $pop3 = Net::POP3->new($self->{'pop_server'}) or die('Failed to connect.');
		my $check_auth = undef;
		if ($self->{'pop_useapop'}) {
			$check_auth = $pop3->apop($self->{'pop_account'},$self->{'pop_password'});
		} else {
			$pop3->user($self->{'pop_account'});
			$check_auth = $pop3->pass($self->{'pop_password'});
		}
		die('Failed to authorize.') if !defined($check_auth);
		my $msglist = $pop3->list();
		foreach my $msgid (keys %$msglist) {
			my ($tmp,$data);
			next if (${$msglist}{$msgid} > $param{'size'} and $param{'size'} > -1); # ƥ᡼륵ĶƤ
			$tmp = $pop3->top($msgid); # إå
			$data = join('',@$tmp);
			$data = &_linefeed($data);
			$data .= "\n" if ($data !~ /\n$/); # ˲Ԥʤ
			$tmp = $self->extract_head($data,'Subject'); # Subject Ф
			next if ($tmp eq '');
			next if ($param{'subj'} ne '' and index($tmp,$param{'subj'}) == -1);
			if ($param{'from'}) {
				my $frm_flag;
				$tmp = $self->extract_head($data,'From'); # From Ф
				foreach (@{$param{'from'}}) {
					$frm_flag = 1 if (index($tmp,$_) > -1);
					last if ($frm_flag);
				}
				next if (!$frm_flag);
			}
			if ($param{'msgid'} ne '') {
				$tmp = $self->extract_head($data,'Message-ID');
				last if ($tmp eq $param{'msgid'}); # Message-ID פȴ
			}
			$tmp = $pop3->get($msgid); # ᡼
			$data = join('',@$tmp) . "\n.\n";
			$data = &_linefeed($data);
			push(@mail,$data); # ǡǼ
			$pop3->delete($msgid) if ($param{'del'}); # ᡼
		}
		$pop3->quit; # ³λ
	};
	die($@) if ($@); # 
	return(@mail); # ᡼뷲֤
	# [ͻ] ==============================================
	# ⥸塼Ȥä POP3 饤ȤäƤߤ褦
	# http://x68000.q-e-d.net/~68user/net/module-pop3.html
	# Perlǥ᡼
	# http://homepage3.nifty.com/hippo2000/perltips/rcvmail.htm
	# =========================================================
}
sub text_parse { # ᡼
	my $self = shift;
	my $data = shift; # ϥѥ᡼
	my $out = { # ϥѥ᡼
		'head' => '',
		'body' => '',
		'boundary' => '',
	};
	if ($data =~ /^(.*?)\n\n(.*?)\n\.\n$/s) {
		$out->{'head'} = $1 . "\n";
		$out->{'body'} = $2;
	}
	if ($out->{'head'} =~ /Content-Type:\s?multipart\/mixed;\n?\s?boundary="(.*?)"/si) {
		$out->{'boundary'} = '--' . $1;
	} else {
		$out->{'boundary'} = '';
	}
	return($out);
}
sub multipart { # multipart 
	my $self = shift;
	my $data = shift; # ϥѥ᡼ ( txet_parse ν )
	my @out; # ϥѥ᡼
	my @parts = split($data->{'boundary'}, $data->{'body'});
	shift @parts; # ǸΥѡȤΤƤ
	require 'mimeutil.pl';
	for (my $i=0;$i<@parts;$i++) {
		if($parts[$i] =~ /(.*?)\n\n(.*)/s){
			my ($head,$body) = ($1,$2);
			$out[$i]->{'head'} = $head;
			$out[$i]->{'type'} = &_part_type($head);
			$out[$i]->{'code'} = &_part_encode($head);
			$out[$i]->{'name'} = &_part_name($head);
			if ($out[$i]->{'code'} =~ /Base64/i) {
				$out[$i]->{'body'} = &mimeutil::bodydecode($body,'b64');
				$out[$i]->{'body'} .= &mimeutil::bdeflush('b64');
			} elsif ($out[$i]->{'code'} =~ /Quoted/i) {
				$out[$i]->{'body'}  = &mimeutil::bodydecode($body,'qp');
				$out[$i]->{'body'} .= &mimeutil::bdeflush('qp');
			} else {
				$out[$i]->{'body'} = $body;
			}
		}
	}
	return(\@out);
}
sub extract_head { # إåȴФ
	my $self = shift;
	my ($head,$elem) = @_; # ϥѥ᡼
	my $out;               # ϥѥ᡼
	require 'mimeutil.pl';
	if ($head =~ /$elem: (.*?)\n([\w|-]*?): /s) { # إå¸ߤ
		$out = $1;
	} elsif ($head =~ /$elem: (.*?)\n$/s) { # إåκǸ¸ߤ
		$out = $1;
	}
	$out = &mimeutil::mimedecode($out, 'EUC'); # ɸ
	$out =~ s/\n\s?//g;
	$out =~ s/\s$//;
	return($out);
}
# ==================================================
# // private functions
# ==================================================
sub _generate_header {
	my %param = @_;
	my $header;
	foreach my $key ( keys(%param) ) {
		next if ($key eq 'Body');
		$header .= $key . ': ' . sb::Language->get->mailtext($param{$key}) . "\n";
	}
	$header .= 'MIME-Version: 1.0' . "\n";
	$header .= 'Content-type: text/plain; charset=' . sb::Language->get->string('parts_mailchar') . "\n";
	$header .= 'Content-Transfer-Encoding: 7bit' . "\n";
	$header .= 'X-Mailer: ' . $sb::PRODUCT . ' ' . $sb::VERSION . "\n";
	$header .= "\n"; # إåȥܥǥζڤ
	require 'mimeutil.pl';
	return &mimeutil::mimeencode($header);
}
sub _linefeed {
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	return($_[0]);
}
sub _part_type { # Content-Type Ƚ
	my $head = shift;
	return($1) if ($head =~ /Content-Type: (.*?);/si);
	return();
}
sub _part_name { # ե̾
	my $head = shift;
	return($1) if ($head =~ /Content-Type:\s?.*?;\n?\s*name="(.*?)"/si);
	return();
}
sub _part_encode { # 󥳡ɥȽ
	my $head = shift;
	return($1) if ($head =~ /Content-Transfer-Encoding: (.*)/si); # by Kei Sato
	return();
}
1;
__END__
